import os
import fileinput, re, string
import pmm_config
import logging


class ExInvalidConfigLine(Exception):
    def __init__(self, line):
        Exception.__init__(self)
        self.line = line

    def __str__(self):
        return "Invalid config line: " + self.line


_logger = logging.getLogger("migration_handler.migration_handler_rc")


instance = None


def get(key = None):
    global instance
    if not instance:
        instance = MigrationHandlerConfig()
    if not key:
        return instance
    return instance[key]


class MigrationHandlerConfig:
    __config = {}

    def __log_info(self, keyname, exception):
        _logger.info("Exception '" + repr(exception) + "' during retrieving key " + keyname + " value: " + str(exception))

    def __init__(self, configName = None):
        if configName is None:
            configName = os.path.join(pmm_config.migration_handler_dir(),"migration_handler-rc")
        self.__parse(configName)

    def __len__(self):
        return len(self.__config)

    def __getitem__(self, key):
        return self.__config[key]

    def __iter__(self):
        return self.iterkeys()

    def iterkeys(self):
        return self.__config.iterkeys()

    def iteritems(self):
        return self.__config.iteritems()

# Maximum number of resolve conflicts iteration during migration
    def get_migration_max_conflict_resolves(self):
        migration_max_conflict_resolves = 2
        try:
            migration_max_conflict_resolves_str = get("MAX_MIGRATION_CONFLICT_RESOLVES")
            migration_max_conflict_resolves = int(migration_max_conflict_resolves_str)
        except (KeyError, ValueError), e:
            self.__log_info("MAX_MIGRATION_CONFLICT_RESOLVES",e)
        return migration_max_conflict_resolves

    def __parse(self, configFile):
        value = re.compile('^(\S+)\s+(.*)$')
        for line in fileinput.input(configFile):
            line = line.rstrip()
            if not line:
                continue
            if line.startswith("#"):
                continue
            data = re.match(value, line)
            if not data:
                raise ExInvalidConfigLine, line
            self.__config[data.group(1)] = string.rstrip(data.group(2))


# [a sort of] test suite
if __name__ == '__main__':
    pc = MigrationHandlerConfig()
    for i, v in pc.iteritems():
        print i + "->"+ v
