import sys
from pmm_api_xml_protocols import TaskStatus, Finished, Working, Starting
import logging
from subproc import NonzeroExitException
import pmmcli_exceptions

mswindows = (sys.platform == "win32")

_logger = logging.getLogger("pmm.migration_handler.remote_service")

class MigrationRemoteService:
    def __init__(self, host, port, login, passwd, session):
        self.host = host
        self.port = port
        self.login = login
        self.passwd = passwd
        self.session = session

    def getSessionPath(self):
        return self.session.getSessionPath()

    def getService(host, port, login, passwd, session):
        import pmm_migr_remote_service_impl
        if mswindows:
            return  pmm_migr_remote_service_impl.MigrationRemoteServiceWin(host, port, login, passwd, session)
        else:
            return  pmm_migr_remote_service_impl.MigrationRemoteServiceUnix(host, port, login, passwd, session)

    getService = staticmethod(getService)

    def sendScout(self, system_type = None):
        raise TypeError('Abstract method `' + self.__class__.__name__ + '.sendScout called')

    def getScoutResult(self):
        raise TypeError('Abstract method `' + self.__class__.__name__ + '.getScoutResult called')

    def removeScout(self):
        raise TypeError('Abstract method `' + self.__class__.__name__ + '.removeScout called')

    def deployAgent(self, agent_name, agent_dir):
        raise TypeError('Abstract method `' + self.__class__.__name__ + '.deployAgent called')

    def getObjectsList(self, agent_name, agent_dir):
        raise TypeError('Abstract method `' + self.__class__.__name__ + '.getObjectsList called')

    def prepareCmdLine(self, filename):
        return self.session.createWorker(filename)

    def runCmdLine(self, cmd):
        try:
            self.session.runWorker(cmd)
        except NonzeroExitException, e:
            if e.exitcode == 10:
                _logger.error(unicode(e))
                raise pmmcli_exceptions.PMMHostNotAccessibleException(u"Host %s is not accessible" % unicode(self.host))
            raise

    def removeData(self, agent_name, agent_dir):
        raise TypeError('Abstract method `' + self.__class__.__name__ + '.removeData called')

    def selectAccounts(self, file_path):
        raise TypeError('Abstract method `' + self.__class__.__name__ + '.selectAccounts called')

    def getIpMap(self, file_path):
        raise TypeError('Abstract method `' + self.__class__.__name__ + '.getIpMap called')

    def clearStatus(self):
        raise TypeError('Abstract method `' + self.__class__.__name__ + '.clearStatus called')

    def getStatusImpl(self, agent_name, agent_dir):
        raise TypeError('Abstract method `' + self.__class__.__name__ + '.getStatusImpl called')

    def getStatus(self, agent_name, agent_dir):
        try:
            status = self.getStatusImpl(agent_name, agent_dir)
            self.session.setBackupCurrentStatus(status)
            return status
        except Exception, x:
            _logger.warning("Exception raised in getStatus: %s" % str(x))
            status = self.session.getBackupCurrentStatus()
            if status is None:
                _logger.debug("Current backup status is not saved. Forming initial status.")
                status = TaskStatus.factory()
                working = Working.factory()
                working.starting = Starting.factory()
                status.working = working
            return status

    def backupImpl(self, agent_name, agent_dir):
        raise TypeError('Abstract method `' + self.__class__.__name__ + '.backupImpl called')

    def backup(self, agent_name, agent_dir):
        try:
            self.__saveBackupStatus(None)
            res = self.backupImpl(agent_name, agent_dir)
            self.__saveBackupStatus(TaskStatus.factory(finished = Finished.factory( status = self.__getBackupStatus(), log_location = None )))
            return res
        except:
            self.__saveBackupStatus(TaskStatus.factory(finished = Finished.factory( status = 'error', log_location = None )))
            raise

    def __saveBackupStatus(self, status):
        try:
            self.session.setBackupFinishedStatus(status)
            _logger.debug(u"Backup status saved to session")
        except Exception, e:
            _logger.warning(u"Unable to save backup status: %s" % unicode(e))
        except:
            _logger.warning(u"Unable to save backup status: unknown error")

    def __getBackupStatus(self):
        status = self.session.getMigrationStatus()
        
        if status == 'success':
            return 'success'
        if status == 'error':
            return 'error'
        _logger.warning("Migration status has unexpected value: %s" % status)
        return 'error'
